@@ -1,8 +1,8 @@
 /*
 **  OSSP sa - Socket Abstraction
-**  Copyright (c) 2001-2003 Ralf S. Engelschall <rse@engelschall.com>
-**  Copyright (c) 2001-2003 The OSSP Project <http://www.ossp.org/>
-**  Copyright (c) 2001-2003 Cable & Wireless Deutschland <http://www.cw.com/de/>
+**  Copyright (c) 2001-2005 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 2001-2005 The OSSP Project <http://www.ossp.org/>
+**  Copyright (c) 2001-2005 Cable & Wireless <http://www.cw.com/>
 **
 **  This file is part of OSSP sa, a socket abstraction library which
 **  can be found at http://www.ossp.org/pkg/lib/sa/.
@@ -89,6 +89,8 @@ typedef int socklen_t;
 #define sa_flush        SA_CONCAT(SA_PREFIX,sa_flush)
 #define sa_recv         SA_CONCAT(SA_PREFIX,sa_recv)
 #define sa_send         SA_CONCAT(SA_PREFIX,sa_send)
+#define sa_sendf        SA_CONCAT(SA_PREFIX,sa_sendf)
+#define sa_id           SA_CONCAT(SA_PREFIX,sa_id)
 #define sa_error        SA_CONCAT(SA_PREFIX,sa_error)
 #endif
 
@@ -154,7 +156,8 @@ typedef enum {
     SA_SYSCALL_READ,
     SA_SYSCALL_WRITE,
     SA_SYSCALL_RECVFROM,
-    SA_SYSCALL_SENDTO
+    SA_SYSCALL_SENDTO,
+    SA_SYSCALL_GETHOSTBYNAME
 } sa_syscall_t;
 
 /* unique library identifier */
@@ -180,7 +183,7 @@ extern sa_rc_t sa_type         (sa_t  *__sa, sa_type_t    __id);
 extern sa_rc_t sa_timeout      (sa_t  *__sa, sa_timeout_t __id, long __sec, long __usec);
 extern sa_rc_t sa_buffer       (sa_t  *__sa, sa_buffer_t  __id, size_t __size);
 extern sa_rc_t sa_option       (sa_t  *__sa, sa_option_t  __id, ...);
-extern sa_rc_t sa_syscall      (sa_t  *__sa, sa_syscall_t __id, void (*__fptr)(), void *__fctx);
+extern sa_rc_t sa_syscall      (sa_t  *__sa, sa_syscall_t __id, void (*__fptr)(void), void *__fctx);
 
 /* socket connection operations */
 extern sa_rc_t sa_bind         (sa_t  *__sa, sa_addr_t  *__laddr);
@@ -207,10 +210,5 @@ extern sa_rc_t sa_sendf        (sa_t  *__sa, sa_addr_t  *__raddr, const char *__
 /* error handling operations */
 extern char   *sa_error        (sa_rc_t __rv);
 
-/* cleanup */
-#if defined(HAVE_CONFIG_H) && !defined(HAVE_SOCKLEN_T)
-#undef socklen_t
-#endif
-
 #endif /* __SA_H__ */
 
